from typing import Optional

from pydantic import BaseModel, model_validator, ConfigDict

from aiosteam.clients.requests_client import RequestsClient
from aiosteam.exceptions.api_errors import NotFound
from aiosteam.steam_models.badges import Badges
from aiosteam.steam_models.games import Game


class UserAvatarModel(BaseModel):
    avatar: str
    avatar_medium: str
    avatar_full: str
    avatar_hash: str


class User(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    steam_id: int
    player_lvl: Optional[int]
    community_visibility_state: int
    profile_state: int
    persona_name: str
    profile_url: str
    avatar: UserAvatarModel
    last_logoff: Optional[int]
    persona_state: int
    real_name: Optional[str]
    primary_clan_id: int
    time_created: int
    persona_state_flags: int
    loc_country_code: Optional[str]
    friends: Optional[list['User']]
    last_played_games: Optional[dict[int, Game]]
    owned_games: Optional[dict[int, Game]]
    user_badges: Optional[Badges]
    relationship: Optional[str]
    friend_since: Optional[int]
    client: RequestsClient

    @model_validator(mode='before')
    def create_avatar_field(cls, inp: dict):
        new_inp = {}
        for field in cls.model_fields:
            replaced = field.replace('_', '')
            value = _value if (_value := inp.get(replaced)) is not None else inp.get(field)
            new_inp[field] = value

        new_inp['avatar'] = inp['avatar'] if isinstance(inp['avatar'], UserAvatarModel) else UserAvatarModel(
            avatar=inp.pop('avatar'), avatar_medium=inp.pop('avatarmedium'), avatar_full=inp.pop('avatarfull'),
            avatar_hash=inp.pop('avatarhash'))

        return new_inp

    @staticmethod
    async def search_user(search: str, client: RequestsClient):
        """Searches for exact match

        Args:
            search (str): steam user. For example 'the12thchairman'
            client (aiosteam.Client): aiosteam.Client
        """
        search_response = await client.request("get", "/ISteamUser/ResolveVanityURL/v1/",
                                               params={"vanityurl": search})

        if search_response["response"]["success"] != 1:
            return search_response["response"]["message"]
        steam_id = search_response["response"]["steamid"]
        return await User.get_user_details(steam_id, client)

    @staticmethod
    async def get_user_details(steam_id: str or int, client: RequestsClient, single=True):
        """Gets user/player details by async_steam ID

        Args:
            steam_id (str or int): Steam 64 ID
            client (aiosteam.Client): aiosteam.Client
            single (bool, optional): Gets one player. Defaults to True. When false, steam_id can be a string of steamids and delimited by a ','

        """
        user_response = await client.request("get", "/ISteamUser/GetPlayerSummaries/v2/",
                                             params={"steamids": steam_id})
        if not user_response["response"]["players"]:
            raise NotFound(f'{steam_id} is not found')
        if single:
            return User(client=client, **user_response["response"]["players"][0])
        else:
            users = []
            for player in user_response["response"]["players"]:
                users.append(User(client=client, **player))
            return users

    async def get_all_info(self):
        await self.get_player_lvl()
        await self.get_user_badges()
        await self.get_last_played_games()
        await self.get_owned_games()
        await self.get_player_bans()
        await self.get_user_friends_list()
        return self

    # await self.get_account_public_info() # not worked on steam side

    async def get_user_friends_list(self) -> list['User']:
        """
        Gets friend list of a user
        """

        friends_list_response = await self.client.request("get", "/ISteamUser/GetFriendList/v1/",
                                                          params={"steamid": self.steam_id})
        self.friends = await self._transform_friends(friends_list_response["friendslist"])
        return self.friends  # noqa this is realy return User object

    async def get_last_played_games(self) -> Optional[dict[int, Game]]:
        """Gets recently played games
        """
        games = {}
        response = await self.client.request("get", "/IPlayerService/GetRecentlyPlayedGames/v1/",
                                             params={"steamid": self.steam_id})
        if response["response"].get('total_count'):
            for game in response["response"]["games"]:
                game.update({"client": self.client, "from_user_id": self.steam_id})
                game_object = Game.model_validate(game)
                games[game_object.app_id] = game_object
        self.last_played_games = games
        return self.last_played_games

    async def get_owned_games(self, include_appinfo=True, include_free_games=True) -> Optional[dict[int, Game]]:
        """Gets all owned games of a user by async_steam id

        Args:
            include_appinfo (bool, optional): Includes app/game info. Defaults to True.
            include_free_games (bool, optional): Includes free games. Defaults to True.
        """
        params = {
            "steamid": self.steam_id,
            "include_appinfo": include_appinfo,
            "include_played_free_games": include_free_games,
        }
        response = await self.client.request("get", "/IPlayerService/GetOwnedGames/v1/", params=params)
        games = {}
        for game in response["response"]["games"]:
            game.update({"client": self.client, "from_user_id": self.steam_id})
            owned = Game.model_validate(game)
            games[owned.app_id] = owned

        if self.last_played_games:
            for app_id, game in self.last_played_games.items():
                if games.get(app_id):
                    games[app_id].playtime_two_weeks = game.playtime_two_weeks

        self.owned_games = games
        return self.owned_games

    async def get_player_lvl(self) -> dict:
        """Gets user async_steam level

        """
        response = await self.client.request("get", "/IPlayerService/GetSteamLevel/v1/",
                                             params={"steamid": self.steam_id})
        self.player_lvl = response["response"]['player_level']
        return response["response"]

    async def get_user_badges(self) -> Badges:
        """Gets user async_steam badges
        """
        response = await self.client.request("get", "/IPlayerService/GetBadges/v1/",
                                             params={"steamid": self.steam_id})
        badges = Badges.model_validate(response["response"])
        self.user_badges = badges
        return badges

    # async def get_community_badge_progress(self, badge_id: int or str) -> dict:
    #     """Gets user community badge progress
    #
    #     Args:
    #         badge_id (int): Badge ID
    #     """
    #     response = await self.client.request("get", "/IPlayerService/GetCommunityBadgeProgress/v1",
    #                                            params={"steamid": self.steam_id, "badgeid": badge_id}, )
    #     return response["response"]

    async def get_account_public_info(self) -> dict:
        """Gets account public info"""
        response = await self.client.request("get", "/IGameServersService/GetAccountPublicInfo/v1",
                                             params={"steamid": self.steam_id})
        return response

    async def get_player_bans(self) -> dict:
        """Gets account bans info
        """
        response = await self.client.request("get", "/ISteamUser/GetPlayerBans/v1",
                                             params={"steamids": self.steam_id})
        return response

    async def _transform_friends(self, friends_list: dict) -> list[Optional['User']]:
        friend_steam_ids = {friend["steamid"]: friend for friend in friends_list["friends"]}
        friends = await self.get_user_details(",".join(friend_steam_ids.keys()), self.client, False)

        for f in friends:
            if str(f.steam_id) in set(friend_steam_ids.keys()):
                friend = friend_steam_ids[str(f.steam_id)]
                f.relationship = friend["relationship"]
                f.friend_since = friend["friend_since"]

        return friends

    async def get_steamid(self, vanity: str) -> dict:
        """Get steamid64 from vanity URL

        Args:
            vanity (str): Vanity URL
        """
        response = await self.client.request("get", "/ISteamUser/ResolveVanityURL/v1", params={"vanityurl": vanity})
        return response["response"]



from aiosteam.clients.requests_client import RequestsClient
from aiosteam.steam_models import User


class Steam:
    """Steam API client"""

    def __init__(self, key: str, headers=None):
        """Constructor for Steam API client"""
        if headers is None:
            headers = {}
        self.client = RequestsClient(key, headers=headers)
        self.__users = User

    async def search_user(self, username: str = None, steam_id: str | int = None) -> User:
        """Searches for exact match

                Args:
                    username (str): steam user. For example 'the12thchairman'
                    steam_id (str): steam id (str or int): Steam 64 ID
        """
        if username:
            user = await self.__users.search_user(username, self.client)
        else:
            user = await self.__users.get_user_details(steam_id, self.client)
        return user



from typing import Optional

from pydantic import BaseModel, model_validator, ConfigDict

from aiosteam.clients.requests_client import RequestsClient


class Game(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)
    client: RequestsClient
    app_id: int
    name: str
    playtime_forever: Optional[int] = None
    img_icon_url: Optional[str] = None
    playtime_two_weeks: Optional[int] = None
    user_stats: Optional[dict] = {}
    user_achievements: Optional[dict] = {}
    from_user_id: Optional[int] = None
    required_age: Optional[int] = None
    is_free: Optional[bool] = None
    dlc: Optional[list['Game']] = None
    detailed_description: Optional[str] = None
    about_the_game: Optional[str] = None
    short_description: Optional[str] = None
    supported_languages: Optional[dict[str, str]] = None
    header_image: Optional[str] = None
    capsule_image: Optional[str] = None
    capsule_imagev5: Optional[str] = None
    website: Optional[str] = None
    pc_requirements: Optional[dict | list] = None
    mac_requirements: Optional[dict | list] = None
    linux_requirements: Optional[dict | list] = None
    player_achievement_statistics: Optional[dict] = {}

    @model_validator(mode='before')
    def create_avatar_field(cls, inp: dict):
        inp["playtime_two_weeks"] = inp.pop('playtime_2weeks', None)
        inp["app_id"] = inp.pop('appid', None) or inp.pop('steam_appid', None)

        return inp

    @staticmethod
    def parse_supported_languages(text) -> dict[str, str]:
        text = text.strip('languages with full audio support')
        text = text.split(', ')
        result = {}
        for row in text:
            if len(row.split('<')) >= 2:
                name = row.split('<')[0]
                result[name] = 'full'
            else:
                result[row] = 'text'
        return result

    async def get_user_stats(self) -> dict:
        """Obtains a user's stats for a specific app, includes only completed achievements
        along with app specific information"""
        response = await self.client.request("get", "/ISteamUserStats/GetUserStatsForGame/v2/",
                                             params={"steamid": self.from_user_id, "appid": self.app_id})
        self.player_achievement_statistics.update(response)
        return self.player_achievement_statistics

    async def get_user_achievements(self) -> dict:
        """Obtains information of the user's achievments in the app
        """
        response = await self.client.request("get", "/ISteamUserStats/GetPlayerAchievements/v1/",
                                             params={"steamid": self.from_user_id, "appid": self.app_id})
        self.user_achievements.update(response)
        return self.user_achievements

    async def get_info_from_shop(self) -> 'Game':
        game = await self.client.get_app_details(app_id=self.app_id)
        self.required_age = int(game.get('required_age'))
        self.is_free = game.get('is_free')
        self.detailed_description = game.get('detailed_description')
        self.about_the_game = game.get('about_the_game')
        self.short_description = game.get('short_description')
        self.supported_languages = self.parse_supported_languages(game.get('supported_languages', ''))
        self.pc_requirements = game.get('pc_requirements')
        self.mac_requirements = game.get('mac_requirements')
        self.linux_requirements = game.get('linux_requirements')
        self.header_image = game.get('header_image')
        self.capsule_image = game.get('capsule_image')
        self.capsule_imagev5 = game.get('capsule_imagev5')
        if dlcs := game.get('dlc', []):
            dlc = []
            for row in dlcs:
                game_dlc = await self.client.get_app_details(app_id=row)
                game_dlc['client'] = self.client
                game_dlc['supported_languages'] = self.parse_supported_languages(
                    game_dlc.get('supported_languages', ''))
                parsed_dlc = Game.model_validate(game_dlc)
                dlc.append(parsed_dlc)
            self.dlc = dlc
        return self

    async def get_all_info(self):
        await self.get_info_from_shop()
        await self.get_user_achievements()
        await self.get_user_stats()
        return self



from pydantic import BaseModel, model_validator


class Badge(BaseModel):
    badge_id: int
    level: int
    completion_time: int
    xp: int
    scarcity: int

    @model_validator(mode='before')
    def reformat_badge_id(cls, inp: dict):
        inp['badge_id'] = inp.pop('badgeid')
        return inp


class Badges(BaseModel):
    badges: list[Badge]
    player_xp: int
    player_level: int
    player_xp_needed_to_level_up: int
    player_xp_needed_current_level: int



import asyncio

from aiosteam import Steam

steam = Steam("B59856FFDC7B2215EE7F5F26913A5E56")

async def some_async_foo():
  user = await steam.search_user("jeygavrus") # also you can use steam user id for searching
  await user.get_all_info()
  await user.owned_games[20920].get_all_info()

  print(user) # list[User]



if __name__ == '__main__':
    asyncio.run(some_async_foo())